package org.phosphoresce.lib.commons.util;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import javax.swing.ImageIcon;

/**
 * リソースユーティリティクラス<br>
 * <br>
 * 当クラスはクラスローダが提供するリソース、ファイルシステム上のリソースを
 * 意識せずにリソースアクセスを行うために設けたユーティリティクラスです。<br>
 *
 * @author Kitagawa<br>
 *
 *<!--
 * 更新日		更新者			更新内容
 * 2009/04/16	Kitagawa		新規作成
 * 2009/12/20	Kitagawa		{@link #getURL(String)}をpublic化
 * 2010/07/05	Kitagawa		isExistResource追加
 * 2011/05/10	Kitagawa		writeText追加
 *-->
 */
public final class ResourceUtil {

	/** バイトバッファサイズ */
	private static final int BYTE_BUFFER = 2048;

	/**
	 * コンストラクタ<br>
	 */
	private ResourceUtil() {
		super();
	}

	/**
	 * 指定されたリソースパスからリソースURLを取得します。<br>
	 * URL変換対象リソースパスはクラスローダから提供されるリソース、
	 * ファイルシステムから提供されるリソースの順序で検索が行われます。<br>
	 * @param name リソースパス
	 * @return 指定されたリソースパスのURLオブジェクト
	 * @throws IOException 指定されたリソース名からURLに変換することが出来なかった場合にスローされます
	 */
	@SuppressWarnings("deprecation")
	public static URL getURL(String name) throws IOException {
		URL url = null;

		url = ResourceUtil.class.getResource(name);
		if (url == null) {
			url = ResourceUtil.class.getResource("/" + name);
		}
		if (url == null) {
			url = Class.class.getResource(name);
		}
		if (url == null) {
			url = Class.class.getResource("/" + name);
		}
		if (url == null) {
			File file = new File(name);
			//if (file.exists()) {
			url = file.toURL();
			//}
		}

		if (url == null) {
			throw new IllegalArgumentException("指定されたリソースパス" + name + "からURLオブジェクトに変換できませんでした");
		}

		return url;
	}

	/**
	 * 指定されたリソースパスからリソースURIを取得します。<br>
	 * URI変換対象リソースパスはクラスローダから提供されるリソース、
	 * ファイルシステムから提供されるリソースの順序で検索が行われます。<br>
	 * @param name リソースパス
	 * @return 指定されたリソースパスのURIオブジェクト
	 * @throws IOException 指定されたリソース名からURLに変換することが出来なかった場合にスローされます
	 * @throws URISyntaxException URLからURIへの変換時に不正なURIパス文字列となった場合にスローされます
	 */
	public static URI getURI(String name) throws IOException, URISyntaxException {
		return getURL(name).toURI();
	}

	/**
	 * 指定されたリソースパスからリソースファイルオブジェクトを取得します。<br>
	 * URI変換対象リソースパスはクラスローダから提供されるリソース、
	 * ファイルシステムから提供されるリソースの順序で検索が行われます。<br>
	 * @param name リソースパス
	 * @return 指定されたリソースパスのファイルオブジェクトオブジェクト
	 * @throws IOException 指定されたリソース名からURLに変換することが出来なかった場合にスローされます
	 * @throws URISyntaxException URLからURIへの変換時に不正なURIパス文字列となった場合にスローされます
	 */
	public static File getFile(String name) throws IOException, URISyntaxException {
		return new File(getURI(name));
	}

	/**
	 * 呼び出し元クラスパッケージから見た相対リソース物理パスを取得します。<br>
	 * @param name リソース名
	 * @return リソース物理パス
	 * @throws IOException 指定されたリソース名からURLに変換することが出来なかった場合にスローされます
	 */
	public static String getPath(String name) throws IOException {
		String packagename;
		try {
			packagename = Class.forName(new Throwable().getStackTrace()[1].getClassName()).getPackage().getName();
		} catch (ClassNotFoundException e) {
			packagename = "java.lang";
		}
		packagename = packagename.replaceAll("\\.", "/");
		packagename = packagename + "/";
		return ResourceUtil.getURL(packagename + name).getFile();
	}

	/**
	 * 指定されたリソースがリソースパス又はファイルシステムに存在するか判定します。<br>
	 * @param name リソースパス
	 * @return 指定したリソースが存在する場合にtrueを返却します
	 * @throws IOException リソースの存在チェックが正しく行えなかった場合にスローされます
	 */
	public static boolean isExistResource(String name) throws IOException {
		try {
			return new File(new URI(getURL(name).toString().replaceAll(" ", "%20"))).exists();
		} catch (URISyntaxException e) {
			throw new IOException(e.getMessage());
		}
	}

	/**
	 * 指定されたリソースパスから入力ストリームを取得します。<br>
	 * @param name リソースパス
	 * @return 入力ストリームオブジェクト
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 */
	public static InputStream getInputStream(String name) throws IOException {
		return getURL(name).openStream();
	}

	/**
	 * 指定されたリソースパスから出力ストリームを取得します。<br>
	 * @param name リソースパス
	 * @return 出力ストリームオブジェクト
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 */
	public static OutputStream getOutputStream(String name) throws IOException {
		return new FileOutputStream(new File(getURL(name).getFile()));
	}

	/**
	 * 指定されたリソースパスからイメージを取得します。<br>
	 * @param name リソースパス
	 * @return イメージオブジェクト
	 * @throws IOException 不正なリソースパスが指定された場合にスローされます
	 */
	public static Image getImage(String name) throws IOException {
		return Toolkit.getDefaultToolkit().getImage(getURL(name));
	}

	/**
	 * 指定されたリソースパスからイメージアイコンを取得します。<br>
	 * @param name リソースパス
	 * @return イメージアイコンオブジェクト
	 * @throws IOException 不正なリソースパスが指定された場合にスローされます
	 */
	public static ImageIcon getImageIcon(String name) throws IOException {
		return new ImageIcon(getImage(name));
	}

	/**
	 * 指定されたリソースパスからテキストを取得します。<br>
	 * @param name リソースパス
	 * @param charset キャラクタセット
	 * @return リソース内に記述されている文字列
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 */
	public static String getText(String name, String charset) throws IOException {
		BufferedReader reader = null;
		if (charset == null) {
			reader = new BufferedReader(new InputStreamReader(getInputStream(name)));
		} else {
			reader = new BufferedReader(new InputStreamReader(getInputStream(name), charset));
		}
		StringBuffer buffer = new StringBuffer();
		while (reader.ready()) {
			buffer.append(reader.readLine());
			if (reader.ready()) {
				buffer.append("\n");
			}
		}
		reader.close();
		return buffer.toString();
	}

	/**
	 * 指定されたリソースパスからテキストを取得します。<br>
	 * @param name リソースパス
	 * @return リソース内に記述されている文字列
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 */
	public static String getText(String name) throws IOException {
		return getText(name, null);
	}

	/**
	 * 指定されたリソースパスに対してテキストを出力します。<br>
	 * @param name リソースパス
	 * @param text テキスト文字列
	 * @param charset キャラクタセット
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 */
	public static void writeText(String name, String text, String charset) throws IOException {
		BufferedWriter writer = null;
		if (charset == null) {
			writer = new BufferedWriter(new OutputStreamWriter(getOutputStream(name)));
		} else {
			writer = new BufferedWriter(new OutputStreamWriter(getOutputStream(name), charset));
		}
		writer.write(text);
		writer.flush();
		writer.close();
	}

	/**
	 * 指定されたリソースパスに対してテキストを出力します。<br>
	 * @param name リソースパス
	 * @param text テキスト文字列
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 */
	public static void writeText(String name, String text) throws IOException {
		writeText(name, text, null);
	}

	/**
	 * 指定されたリソースパスからバイトデータを取得します。<br>
	 * @param name リソースパス
	 * @return リソースデータバイト配列
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 */
	public static byte[] getBytes(String name) throws IOException {
		InputStream inputStream = getInputStream(name);
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		int length = -1;
		byte[] buffer = new byte[BYTE_BUFFER];
		while ((length = inputStream.read(buffer)) != -1) {
			outputStream.write(buffer, 0, length);
		}
		byte[] result = outputStream.toByteArray();

		inputStream.close();
		outputStream.close();

		return result;
	}

	/**
	 * 指定されたリソースパスからシリアライズされたオブジェクトを読み込みます。<br>
	 * @param name リソースパス
	 * @return 復元されたオブジェクト
	 * @throws IOException ストリーム操作時に入出力例外が発生した場合にスローされます
	 * @throws ClassNotFoundException シリアライズオブジェクトのクラスが不明な場合にスローされます
	 */
	public static Object getObject(String name) throws IOException, ClassNotFoundException {
		ObjectInputStream inputStream = new ObjectInputStream(getInputStream(name));
		Object object = inputStream.readObject();
		inputStream.close();
		return object;
	}
}
